#include "stdafx.h"
#include "header.h"

class CMapEditor
{
public:
	int TileRover (void);
private:

	// Functions

	int  FindMouse_ (void);
	int  Menu_ (void);
	int  MenuDisplay_ (void);
	int  MoveMap_ (int iDirection);
	int  Palette_ (void);
	int  PaletteDisplay_ (void);
    int  PointerSet_ (bool bSet);
	int  ScrollMap_ (void);

	// Variables
	
	bool bQuit;
	bool bKeyDown[127];		// Waiting for the user to let go of a key.

	int iTool;				// The current tool that the user is using.
	int iTerrain;			// The terrain that the user is wanting to enter.
	int iLayer;				// The current layer being edited to.

	int iOffsetX;			// How far from the map's X origin the screen is.
	int iOffsetY;			// How far from the map's X origin the screen is.
	int iOffsetXX;			// An additional pixel offset to add to OffsetX to give a true scrolling effect.
	int iOffsetYY;			// An additional pixel offset to add to OffsetY to give a true scrolling effect.
	int iScrollSpeed;		// How quickly to scroll the screen.
	int iX;					// The X Location of the Tile that the mouse is currently hovering over.
	int iY;					// The Y Location of the Tile that the mouse is currently hovering over.
	int iXPos;				// The exact location on the screen of the tile that the mouse is hovering over on the screen.
	int iYPos;				// The exact location on the screen of the tile that the mouse is hovering over on the screen.

	string tPath;
} edit; // end CMapEditor



// Public Functions

int CMapEditor::TileRover (void)
{
	bQuit = false;
    iOffsetX = 0;
    iOffsetY = 0;
	map.Clear();
	iScrollSpeed = 4;
    
    parStrings(txt, "map_editor");

	map.LoadTiles("001");
    
    // Set the tool to the Terrain Tool
    
    iTool = 0;
    PointerSet_(true);
    iTerrain = 4;

	// Main "Tile Rover" Loop

	while(bQuit == false)
	{
		clear(mBuffer);

		// Highlight the tile that the mouse is hovering over.

		map.Display(iOffsetX, iOffsetY, iOffsetXX, iOffsetYY, false, iLayer);
		FindMouse_();
		drawing_mode(DRAW_MODE_TRANS, 0, 0, 0);
		rectfill(mBuffer, iXPos, iYPos, iXPos + giTileX - 1, iYPos + giTileY - 1, makecol(255, 255, 255));
		solid_mode();

		{
			int iLocX = mouse_x + 16;
			int iLocY = mouse_y + 16 - (giTileHeight - giTileY);
            
            // Terrain
            
			if(iTool == 0)
            {
                int iSrcX = (0 * giTileWidth) + giTileWidth;
                int iSrcY = ((iTerrain - 1) * giTileHeight) + giTileHeight;
            
                if(iTerrain != 0)
                {
                    rect(mBuffer, iLocX - 1, iLocY - 1 + (giTileHeight/3), iLocX + giTileWidth, iLocY + giTileHeight, makecol(0,0,0));
                    masked_blit(map.mTiles, mBuffer, iSrcX, iSrcY, iLocX, iLocY, giTileWidth, giTileHeight);
                }
                else
                    rectfill(mBuffer, iLocX - 1, iLocY - 1 + (giTileHeight/3), iLocX + giTileWidth, iLocY + giTileHeight, makecol(0,0,0));
            };
            
            // Units
            
			if(iTool == 1)
            {
                masked_blit(rank[1].mUnit, mBuffer, giUnitSourceX, giUnitSourceY, iLocX - giUnitOffsetX, iLocY - giUnitOffsetY + (giTileHeight/3), giUnitWidth, giUnitHeight);
            };                
		};

		// Display Everything.

		displayAll();
        
        // Make sure that the user isn't outside the bounds of the map.
        
        if(iOffsetX > map.iSizeX - giTilesWide)
            iOffsetX = map.iSizeX - giTilesWide;
        if(iOffsetY > map.iSizeY - giTilesHigh)
            iOffsetY = map.iSizeY - giTilesHigh;
		
		// Map Scrolling
		
		ScrollMap_();
		
		if(mouse_x > (SCREEN_W - giTileX) && iOffsetX < map.iSizeX - giTilesWide
			|| key[KEY_RIGHT] && iOffsetX < map.iSizeX - giTilesWide)
			MoveMap_(dRIGHT);
		if(mouse_x < giTileX && iOffsetX > 0
			|| key[KEY_LEFT] && iOffsetX > 0)
			MoveMap_(dLEFT);
		if(mouse_y > (SCREEN_H - giTileY) && iOffsetY < map.iSizeY - giTilesHigh
			|| key[KEY_DOWN] && iOffsetY < map.iSizeY - giTilesHigh)
			MoveMap_(dDOWN);
		if(mouse_y < giTileY && iOffsetY > 0
			|| key[KEY_UP] && iOffsetY > 0)
			MoveMap_(dUP);
		
        // Tool Selection
            
        if(keyCheck(KEY_T))
        {
            iTool = 0;
            PointerSet_(true);
        };
        if(keyCheck(KEY_U))
        {
            iTool = 1;
            PointerSet_(true);
        };
        if(keyCheck(KEY_E))
        {
            iTool = 2;
            PointerSet_(true);
        };
        
		// Tool Input

		if(mouse_b & 1)
        {
            if(iTool == 0)
                map.iTerrain[iLayer][iX + iOffsetX][iY + iOffsetY] = iTerrain;
            if(iTool == 1)
                map.iGeneric[iX + iOffsetX][iY + iOffsetY] = 1;
            if(iTool == 2)
                map.iGeneric[iX + iOffsetX][iY + iOffsetY] = 0;            
        };
		if(mouse_b & 2)
        {
            if(iTool == 0)
                iTerrain = map.iTerrain[iLayer][iX + iOffsetX][iY + iOffsetY];
        };
        
        // Tile/Unit/Object Selection

		if(keyCheck(KEY_MINUS_PAD) && iTerrain > 0 || keyCheck(KEY_MINUS) && iTerrain > 0)
        {
            if(iTool == 0)
                iTerrain--;
        };
		if(keyCheck(KEY_PLUS_PAD) && iTerrain < 25 || keyCheck(KEY_EQUALS) && iTerrain < 25)
        {
            if(iTool == 0)
                iTerrain++;
        };
        
        // Other Input
        
		if(keyCheck(KEY_G))
			map.ToggleGrid();
        
        if(keyCheck(KEY_F1))
        {
            PointerSet_(false);
            htmManual("index");
            parStrings(txt, "map_editor");
            PointerSet_(true);
        };
		if(keyCheck(KEY_ESC))
        {
            PointerSet_(false);
			Menu_();
            PointerSet_(true);
        };
	};
    
    PointerSet_(false);

	return 0;
};



// Private Functions

int CMapEditor::FindMouse_ (void)
{
	// Finds which tile the mouse is hovering over and
	// updates the variables accordingly.  This is done
	// by simply diving the mouse coordinates by the
	// size of an individual tile.

	iX = mouse_x / giTileX;
	iY = mouse_y / giTileY;
	iXPos = iX * giTileX;
	iYPos = iY * giTileY;

	return 0; // Success
};


int CMapEditor::Menu_ (void)
{
	// The system menu for the map editor, displaying basic
	// functions and giving access to other sub-editors.

	keyReset();

	// Set up the menu buttons and text.

	MenuDisplay_();

	// Wait for the user to make a selection.

	while(!butCheck(0))
	{
		clear(mBuffer);
		map.Display(iOffsetX, iOffsetY, iOffsetXX, iOffsetYY, false, iLayer);
		
		drawing_mode(DRAW_MODE_TRANS, 0, 0, 0);
		rectfill(mBuffer, 164, 48, 475, 431, makecol(0, 0, 0));
		solid_mode();
		displayAll();

        if(butCheck(1))
        {
            guiKillAll();
            mapProperties();
            MenuDisplay_();
        };        if(butCheck(6))
        {
            guiKillAll();
            htmManual("index");
            parStrings(txt, "map_editor");
            MenuDisplay_();
        };
		if(butCheck(7))
        {
            bQuit = true;
            break;
        };
		if(keyCheck(KEY_ESC))
			break;
	}; // end while

	keyReset();

	// Clean up.

	butKillAll();
	txtKillAll();

	return 0;
};


int CMapEditor::MenuDisplay_ (void)
{
	butNew(1, 192, 60, "big_01", "", "", 0);
	txtNew(1, 320, 65, txt.Get("prop_ed"), 2, -1, alignCENTER, 0);
	butNew(2, 192, 100, "big_01", "", "", 0);
	txtNew(2, 320, 105, txt.Get("save"), 2, -1, alignCENTER, 0);
	butNew(3, 192, 140, "big_01", "", "", 0);
	txtNew(3, 320, 145, txt.Get("restore"), 2, -1, alignCENTER, 0);
	butNew(4, 192, 180, "big_01", "", "", 0);
	txtNew(4, 320, 185, txt.Get("faction_ed"), 2, -1, alignCENTER, 0);
	butNew(5, 192, 220, "big_01", "", "", 0);
	txtNew(5, 320, 225, txt.Get("prep_ed"), 2, -1, alignCENTER, 0);
	butNew(6, 192, 260, "big_01", "", "", 0);
	txtNew(6, 320, 265, txt.Get("help"), 2, -1, alignCENTER, 0);
	butNew(7, 192, 300, "big_01", "", "", 0);
	txtNew(7, 320, 305, txt.Get("quit"), 2, -1, alignCENTER, 0);

	butNew(0, 192, 380, "big_01", "", "", 0);
	txtNew(0, 320, 385, txt.Get("continue"), 2, -1, alignCENTER, 0);

	return 0; // Success
};

int CMapEditor::MoveMap_ (int iDirection)
{
	// Moves the map in a given direction.
	
	static int iWaiterX;
	static int iWaiterY;
	
	if(iDirection == dRIGHT && iWaiterX > 0 || iDirection == dLEFT && iWaiterX > 0)
		iWaiterX--;
	if(iDirection == dUP && iWaiterY > 0 || iDirection == dDOWN && iWaiterY > 0)
		iWaiterY--;
	
	// Set up the scrolling
	
	if(iDirection == dRIGHT && iOffsetXX == 0 && iWaiterX == 0)
	{
		iOffsetXX = -1 * (giTileX / iScrollSpeed);
		iWaiterX = giTileX / (-1 * iOffsetXX);		
	};

	if(iDirection == dLEFT && iOffsetXX == 0 && iWaiterX == 0)
	{
		iOffsetXX = giTileX / iScrollSpeed;
		iWaiterX = giTileX / iOffsetXX;
	};	

	if(iDirection == dDOWN && iOffsetYY == 0 && iWaiterY == 0)
	{
		iOffsetYY = -1 * (giTileY / iScrollSpeed);
		iWaiterY = giTileY / (-1 * iOffsetYY);
	};	

	if(iDirection == dUP && iOffsetYY == 0 && iWaiterY == 0)
	{
		iOffsetYY = giTileY / iScrollSpeed;
		iWaiterY = giTileY / iOffsetYY;
	};
	
	return 0; // Success
};

int CMapEditor::PointerSet_ (bool bSet)
{
    // Sets the mouse cursor to whatever the current tool is.
    // Send FALSE to reset the cursor to the default.
    
    if(bSet == true)
    {
        if(iTool == 0)
            mouseGFX("terrain_tool_01");
        if(iTool == 1)
            mouseGFX("unit_tool_01");
        if(iTool == 2)
            mouseGFX("erase_tool_01");        
    }
    else
        mouseGFX("pointer_01");
    
    return 0; // Success
};

int CMapEditor::ScrollMap_ (void)
{
	// Handles the actual pixel scrolling set up by MoveMap_
	
	if(iOffsetXX > 0)
		iOffsetXX += giTileX / iScrollSpeed;
	if(iOffsetXX < 0)
		iOffsetXX -= giTileX / iScrollSpeed;
	if(iOffsetYY > 0)
		iOffsetYY += giTileY / iScrollSpeed;
	if(iOffsetYY < 0)
		iOffsetYY -= giTileY / iScrollSpeed;
	
	if(iOffsetXX >= giTileX)
	{
		iOffsetX --;
		iOffsetXX = 0;
	};
	if(iOffsetXX <= -1 * giTileX)
	{
		iOffsetX ++;
		iOffsetXX = 0;
	};
	if(iOffsetYY >= giTileY)
	{
		iOffsetY --;
		iOffsetYY = 0;
	};
	if(iOffsetYY <= -1 * giTileY)
	{
		iOffsetY ++;
		iOffsetYY = 0;
	};
	
	return 0; // Success
};

// Function used for calling the map editor routines.

int editMap (void)
{
	edit.TileRover();

	return 0; // Success
};
