#include "stdafx.h"
#include "header.h"

// class CMap's functions.

int CMap::Clear (void)
{
	// Clears all the map data so that "artifacts" from old maps
	// won't show up when a new map is loaded.

	ClearHeader_();
	ClearTerrain_();

	return 0;	// Success
};

int CMap::ClearHeader_ (void)
{
	// Clears the header data.

	tName = "";
	tAuthor = "";
	tDescription = "";
	iMapVer = 0;
	iSizeX = giTilesWide;
	iSizeY = giTilesHigh;
	iTileSet = 0;

	return 0;	// Success
};

int CMap::ClearTerrain_ (void)
{
	// Clears the terrain data

	for(int layer = 0; layer < 4; layer++)
	{
		for(int x = 0; x < maxWIDTH; x++)
		{
			for(int y = 0; y < maxHEIGHT; y++)
			{
				if(layer == 0)
					iTerrain[layer][x][y] = 1; // If it's the base layer, set the default tile.
				else
					iTerrain[layer][x][y] = 0; // Otherwise, set an "empty" null tile.
			}; // end x for
		}; // end y for
	}; // end layer for
    
    // Clear the generic unit data

    for(int x = 0; x < maxWIDTH; x++)
	{
		for(int y = 0; y < maxHEIGHT; y++)
		{
			iGeneric[x][y] = 0;
		}; // end x for
    }; // end y for    
    
	// Now, destory and clear the bitmap to avoid memory leaks.

	destroy_bitmap(mTiles);
	mTiles = NULL;

	return 0;	// Success
};

int CMap::Display (int iX, int iY, int iOffsetX, int iOffsetY, bool bOjectsOn, int iWorkingLayer)
{
	// Displays the map to the screen buffer bitmap.

	for(int l = 0; l < giTerrainLayers; l++)
	{
		for(int y = -1; y < giTilesHigh + 1; y++)
		{
			for(int x = -1; x < giTilesWide + 1; x++)
			{
				int iLocX = x * giTileX;
				int iLocY = ((y) * giTileY) - (giTileHeight - giTileY);
				int iSrcX = (0 * giTileWidth) + giTileWidth;
				int iSrcY = ((iTerrain[l][x + iX][y + iY] - 1) * giTileHeight) + giTileHeight;
				if(iTerrain[l][x + iX][y + iY] != 0)
					masked_blit(mTiles, mBuffer, iSrcX, iSrcY, iLocX + iOffsetX, iLocY + iOffsetY, giTileWidth, giTileHeight);
			}; // end x for
		}; // end y for
	}; // end l for
    
    // Display Gird if Needed
    
    if(bGrid)
		Grid(iOffsetX, iOffsetY);

    // Displays Units Starting Points (Editor Only)
    
	for(int y = -1; y < giTilesHigh + 1; y++)
	{
		for(int x = -1; x < giTilesWide + 1; x++)
		{
            int iUnitHere = iGeneric[x + iX][y + iY];
			if(iUnitHere != 0 && x > -1 && y > -1)
            {
                int iLocX = (x * giTileX) - giUnitOffsetX;
                int iLocY = (((y) * giTileY) /*- (giTileHeight - giTileY)*/) - giUnitOffsetY;
                int iSrcX = giUnitSourceX;
                int iSrcY = giUnitSourceY;
				masked_blit(rank[iUnitHere].mUnit, mBuffer, iSrcX, iSrcY, iLocX + iOffsetX, iLocY + iOffsetY, giUnitWidth, giUnitHeight);
            };
		}; // end x for
    }; // end y for

	return 0; // Success
};

int CMap::Grid (int iX, int iY)
{
	// Displays a grid. iX and iY are the offset from the screen
	// origin for when you want smooth scrolling.
	
	drawing_mode(DRAW_MODE_TRANS, 0, 0, 0); // Turn on transparency
	
	for(int y = -1; y < giTilesHigh + 1; y++)
	{
		for(int x = -1; x < giTilesWide + 1; x++)
		{
			line(mBuffer, (x * giTileX) + iX, (y * giTileY) + iY, ((x + 1) * giTileX) + iX - 1, (y * giTileY) + iY, makecol(0, 0, 0));
			line(mBuffer, (x * giTileX) + iX, (y * giTileY) + iY + 1, (x * giTileX) + iX, ((y + 1) * giTileY) + iY - 1, makecol(0, 0, 0));
		}; // end x for
	}; // end y for
	
	solid_mode(); // Turn off transparency
	
	return 0; // Success
};	

int CMap::LoadTiles (string tFileName)
{
	// This function will load the map tiles to be used.  The tiles
	// may be stored in png OR bmp format, so we'll try both.

	string tFileTry = sys_tPath + sys_tTilePath + tFileName + ".png";
	
	if(!exists(tFileTry.c_str()))
	{
		tFileTry = sys_tPath + sys_tTilePath + tFileName + ".bmp";
		if(!exists(tFileTry.c_str()))//file_exists(tFileTry.c_str(), 0, NULL))
		{
			char cError[250];
			sprintf(cError, "Cannot find the map tiles requested.\n%s does not appear to exist.\n\nCurrently, the engine is not able to handle this error.", tFileTry.c_str());
			allegro_message(cError);
			exit(0);
		};
	};

	mTiles = load_bitmap(tFileTry.c_str(), NULL);
	colorReplace(mTiles, makecol(255, 128, 255), makecol(255, 0, 255));

	return 0; // Success
};

int CMap::ToggleGrid (void)
{
	// Toggles the grid display on or off.
	
	if(bGrid)
		bGrid = false;
	else
		bGrid = true;
	
	return 0; // Success
};
